/* lsngp.c *************************************** UPDATED: 2000-06-04 21:28 TT
 *
 * Copyright   : (C) 2000 Tormod Tjaberg
 * Author      : Tormod Tjaberg (tjaberg@online.no)
 * Created     : 2000-04-29 
 * Description : LiSt contents og Neo Geo Pocket rom dump
 *
 * Note:
 * Information gleaned from various sources specially Anarko & Judge
 *
 * If you have any other info about the header please contact me.
 *
 * Changes:
 * 1.10 : 2000-06-04 : Added NGP header version number.
 *                     Code should now work on Little & Big endian machines.
 *                     Header read code is no longer machine word dependant.
 * 1.01 : 2000-04-30 : Added size of rom 
 * 1.00 : 2000-04-29 : Initial release
 */
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include <assert.h>

#define PROGRAM_NAME "lsngp"
#define PROGRAM_VER  "1.10"

/* Clever little option mimic wildcard expansion from
 * shell ONLY available with the Zortech compiler ver 3.0x and upwards
 */
#if defined(__ZTC__) 
extern int _cdecl __wildcard; int *__wild = &__wildcard;
#endif

char *Usage[] =
{
   PROGRAM_NAME ", ver " PROGRAM_VER " -- Copyright (C) 2000 Tormod Tjaberg",
   "",
   "Usage: " PROGRAM_NAME " <filename(s)>",
   "",
   "Displays header information from a NeoGeo Pocket rom dump",
#if defined(__ZTC__) || defined(__DJGPP__)
   "Wildcards in the filename are allowed.",
#endif
   NULL
};

/* Abort()
 *
 * Input  : pMessage - Message to be displayed with the format of standard printf!
 * Returns: Nothing, but instead it exits to operating system
 *
 * Terminate program, return to OS with a suitable return value.
 * exit() will close all open files.
 */
void Abort(char *pMessage, ...)
{
   va_list argp;

   fprintf(stderr, PROGRAM_NAME ": ");
   va_start(argp, pMessage);
   vfprintf(stderr ,pMessage, argp);
   va_end(argp);
   fprintf(stderr, "\n");
   exit(EXIT_FAILURE);  /* exit & tell operating system that we've failed */

}

/* Same as Abort we don't exit... 
 */
void Warning(char *pMessage, ...)
{
   va_list argp;

   fprintf(stderr, PROGRAM_NAME ": ");
   va_start(argp, pMessage);
   vfprintf(stderr ,pMessage, argp);
   va_end(argp);
   fprintf(stderr, "\n");
}

/* This is how the structure is defined in the NGPC cart image
 * however to make this work on a non intel machine we need
 * to do a few tricks...
 */
typedef struct
{
   char Copyright[28];  
   unsigned long ProgCount; /* Code starts at 0x200000 */
   unsigned short GameNum;
   unsigned char Version;
   unsigned char Type;
   char Title[12];
   char Filler[16];   /* should always be 0 */
} NGP_HDR;

unsigned int Get_Intel_U_Short(FILE *fp)
{
   unsigned int us;

   us = (unsigned int) (fgetc(fp) & 0xff);
   us |= ((unsigned int) (fgetc(fp) & 0xff) << 0x08);

   return us;
}

unsigned long Get_Intel_U_Long(FILE *fp)
{
   unsigned long ul;

   ul = (unsigned long ) (fgetc(fp) & 0xff);
   ul |= ((unsigned long ) (fgetc(fp) & 0xff) << 0x08);
   ul |= ((unsigned long ) (fgetc(fp) & 0xff) << 0x10);
   ul |= ((unsigned long ) (fgetc(fp) & 0xff) << 0x18);

   return ul;
}

/* 
 * Read the header byte by byte making sure
 * all endianness is cared for.
 *
 * 0 - OK
 * else error
 */
int Read_NGP_Hdr(NGP_HDR *pNGP_Hdr, FILE *fp)
{
   if (fread(&pNGP_Hdr->Copyright, sizeof(char), 28, fp) != 28)
      return 1;

   pNGP_Hdr->ProgCount = Get_Intel_U_Long(fp);
   pNGP_Hdr->GameNum = Get_Intel_U_Short(fp);
   pNGP_Hdr->Version = fgetc(fp);
   pNGP_Hdr->Type = fgetc(fp);

   if (fread(&pNGP_Hdr->Title, sizeof(char), 12, fp) != 12)
      return 2;

   if (fread(&pNGP_Hdr->Filler, sizeof(char), 16, fp) != 16)
      return 3;

   return 0;
}

int main(int argc, char *argv[])
{
   FILE *fp;
   char *pFileSpec;
   int i;
   int j;
   long l;
   int Mbit;
   NGP_HDR NGP_Hdr;

   if (argc < 2)
   {
      i = 0;
      while (Usage[i] != NULL)
         puts(Usage[i++]);
      return 0;
   }

   for (j = 1; j < argc; j++)
   {
      pFileSpec = argv[j];

      if ((fp = fopen(pFileSpec, "rb")) == NULL) 
      {
         Warning("unable to open input file: %s\n", pFileSpec);
         continue;
      }

      memset(&NGP_Hdr, 0, sizeof(NGP_HDR));
      if ((i = Read_NGP_Hdr(&NGP_Hdr, fp)) != 0)
      {
         fclose(fp);
         Warning("error reading NGP header in file: %s (#%d)", pFileSpec, i);
         continue;
      }

      fseek(fp, 0, SEEK_END);
      l = ftell(fp);

      /* close file */
      fclose(fp);
      
      /* Sanity Check
       * . Title has to be printable
       * . Code has to start past 0x200000
       * . Type must be 0x10 (Colour) or 0x00 (Black/White)
       * . Filler bytes must be 0
       */
      if (!isprint(NGP_Hdr.Title[0]) || NGP_Hdr.ProgCount < 0x200000 ||
          NGP_Hdr.Filler[0] != 0 || NGP_Hdr.Filler[1] != 0)
      {
         Warning("%s: is NOT a valid NGP rom", pFileSpec);
         continue;
      }

      if (NGP_Hdr.Type != 0x10 && NGP_Hdr.Type != 0x0)
      {
         Warning("%s: header has invalid type: 0x%x", pFileSpec, NGP_Hdr.Type);
         continue;
      }

      Mbit = (l / 1024) / 128;

      printf("%s: %s: \"%s\" (#%d) %s %d Mbit, Ver %d\n", PROGRAM_NAME, pFileSpec, NGP_Hdr.Title, 
              NGP_Hdr.GameNum, NGP_Hdr.Type == 0x10 ? "Colour" : "Black/White", Mbit, NGP_Hdr.Version);
   }

   return 0;
}
